% function [outrays,outvals,outweights]=LapQCW(level,creal,cimag, plotmth)
% -------------------------------------------------------------------------
%
% level speficies the number of times to apply the polynomial
% cval=creal + i*cimag  ***should be in the main cardiod***
% plotmth(2) is a character
%      -'h'- height: plots measure in 3D
%      -'b'- bubble: plots measure in 2D
%      -'d'- deriv : plots 2|z| 
%      -'u'- nu : plot 
%      -'r'- raycheck:plots exterior rays as height
%      -'c'- the cummulative distribution function for the measure
%      -'a'- all:  spits out all plots 
%      -'n'- none(default)turns off plot, useful as often a helper func.
%   if you would like to specify two of the three plot methods feel free
%   to do so 
%
% Example [r,v,w]=LapQCW(5,.2,0,'br']); 
%
% Does the set up work for computing the laplacian on a Quasicircle
% uses a discrete approximation to the conformal measure as the weights 


function [outrays,outvals,outwghts]=LapQCW2(level,creal,cimag,plotmth)
% This function generates the a graph approximation to the julia set for
% the specified c-value at the specified level.  The information returned 
% is the exterior rays, complex plane locations, and discrete weights for
% the points. 

% We generate our graph approximation by taking pre-images of the fixed
% point P=z^2+c that occurs on the julia set. Level 0 is precisely that
% point. Level m consists of all the preimages of points at level m-1. 

% Weights are assigned to be the discrete approximation to the conformal
% measure. W(z)=W(P(z))*|z|^alpha.  Where alpha is the haussdorf measure
% of the julia set (assuming that the julia has zero area). 

if nargin<3 
    error('I need more info! please specify level, creal and cimag')
elseif nargin==3 
    plotmth='none';
end

% Initialize Parameters
c=creal+sqrt(-1)*cimag;

% Assign the correct value to alpha 
% if you would like to use a c value not listed here you can use my func's 
% summer and SumMaster to estimate alpha yourself.  Atomating this process 
% might be a worthwhile addition to the code. 

if(c==.2); 
    alpha = 1.025725; % Approximate Haussdorf dimmension of the set for c=2;
elseif (c==.33-.25*sqrt(-1))
    alpha = 1.1735; 
elseif (c==.25)
    alpha = 2; 
elseif(c==0||c==-2);  
    alpha=1;          % Dimension of boring Julia Sets 
else 
    fprintf('Using alpha=1.026, likely a bad guess, please  adjust the code to include the Haussdorf dimmension of your julia set\r')
    alpha=1.026;
end 
numpoints = 2^level;

% Initialize Arrays

% The first row in each array holds the values for the most recently
% completed iteration, and the second row the values that are being 
% created from the previous
rays    = zeros(2, numpoints); % Numerator values of the external rays
values  = zeros(2, numpoints); % Point values
weights = zeros(2, numpoints); % Given by the conformal measure
imgidx  = zeros(2, numpoints); % Recordes the index of the image of a point

% Set values for the 0th iteration
% --------------------------------------------
if (level>=0)
    %rays(1,1) =[0];  %okay it's already 0 so I'm not running this line
    values(1,1)=max(roots([1 -1 c])); % this is the fixed point we want
    weights(1,1)=1; % yep, this is arbitrary 
else
    error('Sorry cannot go backward-anywise it would be rather boring, the images of a fixed point....')
end 

% The first iteration (it's unique--only one new preimage!)
% ---------------------------------------------------------------
if ( level>=1 )
    % rays(2,1)   = 0 ; all ready set so don't run this line
    rays(2,2)   = 1 ;
    values(2,1) = values(1,1); 
    weights(2,1) = weights(1,1)*abs(2*values(2,1))^(-alpha);
    imgidx(2,1) = 1;
    
    helper = roots([1 0 c-values(1,1)]);
    I=(helper>=values(1,1)*[.99;.99]&helper<=values(1,1)*[1.01;1.01]);  %pick the root we haven't already used
    values(2,2)  = helper(~I);
    weights(2,2) = weights(1,1)*abs(2*values(2,2))^(-alpha);
    imgidx(2,2)  = 1;

    % Update Arrays
    values(1,:) = values (2,:);
    weights(1,:)= weights(2,:);
    imgidx(1,:) = imgidx (2,:);
    rays(1,:)   = rays   (2,:); 

else 
    error('Out of curiousity, what would a fractional interation be?')
end

clear C helper;  % clean up

% Make a middle axis for dividing points about, may run into issues for
% high iterations
m=(imag(values(1,1))-imag(values(1,2)))/(real(values(1,1))-real(values(1,2)));
if(m==0)
    center=[values(1,1)+.5,values(1,1),values(1,2),values(1,2)-.5];
else
    if(level<8)
     center=[real(values(1,1))+.2, values(1,1),.1+sqrt(-3)*imag(values(1,1)),...
        -.1+sqrt(-3)*imag(values(1,2)),values(1,2),real(values(1,2))-.2];
    else
      center=[real(values(1,1))+.2,real(values(1,1))+.1,values(1,1),.6+sqrt(-3)*imag(values(1,1)),...
          .1+sqrt(-3)*imag(values(1,1)),-.1+sqrt(-3)*imag(values(1,2)),...
          -.6+sqrt(-3)*imag(values(1,2)), values(1,2),real(values(1,2))-.1,real(values(1,2))-.2];
       fprintf('Check Ray Angles\r');
    end
end


% The later iterations 
% --------------------------------------------------

for i=2:level
   % Keep Old Information
   rays(2,:)    = 2*rays(1,:); % As the denominator increases by 2
   % values(2,:)  = values(1,:); %redundent given our update arrays process
   % imgidx(2,:)  = imgidx(1,:);

   %Generate new information
        
   % Calculate new values, the sqrts of the old

   newvals= [sqrt(values(1,2^(i-2)+1:2^(i-1))-c),-sqrt(values(1,2^(i-2)+1:2^(i-1))-c)];
   if (i<7) 
        out=interp1(real(center),imag(center),real(newvals),'linear');
   else 
        out=interp1(real(center),imag(center),real(newvals),'linear');
   end
   
   helper=imag(newvals)>out;
   values(2,2^(i-1)+1:2^i) = newvals; % save organization 
   
   % Calculate new rays
   newrays = (helper).*[rays(1,2^(i-2)+1:2^(i-1)),rays(1,2^(i-2)+1:2^(i-1))];
   newrays = newrays + (~helper).*[rays(1,2^(i-2)+1:2^(i-1))+2^(i-1),...
                                          rays(1,2^(i-2)+1:2^(i-1))+2^(i-1)];
   
   rays(2,2^(i-1)+1:2^i) = newrays;
   
    
   clear helper 

   % Calculate new weights
   imgidx(2,2^(i-1)+1:2^i) = [2^(i-2)+1:2^(i-1),2^(i-2)+1:2^(i-1)];  % Remember images
   weights(2,1:2^(i)) = weights(1,imgidx(2,1:2^(i))).*((2*abs(values(2,1:2^(i)))).^(-alpha));
   
   % Update Arrays
   values(1,:) = values (2,:);
   weights(1,:)= weights(2,:);
   imgidx(1,:) = imgidx (2,:);
   rays(1,:)   = rays   (2,:); 
   
end       

[outrays,I]  =  sort(rays(2,:));  % Do I need to normalize rays??
outvals  =  values(2,I);
outwghts =  weights(2,I); 
outwghts = outwghts/sum(outwghts); % Enforce normalization on weights

% All the Plotting stuff 
% ------------------------------------------------------------------------
if (plotmth=='n')
    return;
end

for i=1:length(plotmth)
    if (plotmth(i)=='h'||plotmth(i)=='a') 
        % Height plot
        % uses the measure as the height
        scatter3(real(outvals),imag(outvals),outwghts,4,outwghts,'*')
        prettyify;
        colorbar;
        title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='b'||plotmth(i)=='a') 
        % Bubble Plot
        % use measures as the size of the weight
        figure;
        scatter(real(outvals),imag(outvals),6*numpoints*outwghts,outwghts,'filled');
        prettyify
        axis equal
        %title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='d'||plotmth(i)=='a') 
        % Derivative Plot
        % use 2|z| as the size of the weight
        figure;
        scatter(real(outvals),imag(outvals),20*abs(outvals),abs(outvals),'filled');
        prettyify
        axis equal
        % title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='u'||plotmth(i)=='a') 
        % Nu Plot
        % plot ray and measure as height
        figure;
        plot(outrays/(max(outrays)+1),outwghts,'k');
        % title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='r'||plotmth(i)=='a') 
        % Check rays
        % plot rays as height, if rays are computed incorrectly 
        % you can expect to see points connected to the opposite side of
        % the quasicircle--can happen on asymmetric quasicircles
        figure;
        plot3(real(outvals),imag(outvals), outrays)
        title('Exterior ray angle check')
    end
    
    if(plotmth(i)=='c'||plotmth(i)=='a')
        % Cumulative Distribution Function Plot
        figure;
        plot(outrays/max(outrays),cumsum(outwghts)/sum(outwghts),'r.','MarkerSize',1)
        title('Normalized Cumulative Sum of Conformal Measures');
        xlabel('Exterior Ray')
    end
end
